<?php

namespace App\Http\Controllers;

use App\Models\Student;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
//use SimpleSoftwareIO\QrCode\Facades\QrCode;
use Illuminate\Support\Facades\Mail;
use Endroid\QrCode\QrCode;
use Endroid\QrCode\Writer\PngWriter;
use PKPass\PKPass;
use Firebase\JWT\JWT;
use Event_tickets\Event;
use Event_tickets\Production;
use Event_tickets\Venue;




class StudentController extends Controller
{
    public function index()
    {
        
        $students = Student::paginate(10); 
        
        return view('students.index', compact('students'));
    }

    public function create()
    {
        return view('students.create');
    }

    public function store(Request $request)
    {
        $validated = $request->validate([
            'name' => 'required|string',
            'university_name' => 'required|string',
            
            'email' => 'required|email|unique:students,email',
            'photo' => 'required|image|mimes:jpeg,png,jpg|max:2048',
            'passport_photo' => 'required|mimes:jpeg,png,jpg,pdf|max:2048', 
        ]);

        $photoPath = $request->file('photo') ? $request->file('photo')->store('photos', 'public') : null;
        $passportPhotoPath = $request->file('passport_photo') ? $request->file('passport_photo')->store('passport_photos', 'public') : null;

        $student = Student::create([
            'name' => $validated['name'],
            'email' => $validated['email'],
            'photo' => $photoPath,
            'passport_photo' => $passportPhotoPath,
            'hotel_name' => $request->hotel_name,
            'university_name' => $request->university_name,
            'conference_date' => $request->conference_date,
        ]);

        $this->sendEmailToStudent($student);

        return redirect()->route('admin.students.index')->with('success', 'Student created successfully and email sent.');
    }

    public function show(Student $student)
    {
        $link = route('students.show', $student->id);  // Replace this with your link
        // Define the path to save the QR code
$directory = public_path('qr-codes'); // Uses Laravel's public_path helper
$filePath = $directory . '/'.$student->id ;

// Create the directory if it does not exist
if (!file_exists($directory)) {
    mkdir($directory, 0755, true); // 0755 is the permission set for the directory
}
        $qrCode = new QrCode($link);
        $writer = new PngWriter();
        $result = $writer->write($qrCode);
        
        // Save the QR code to a file
        $filePath = public_path('qr-codes/student-' . $student->id . '.png');
    file_put_contents($filePath, $result->getString());

        
        //$qrCode = base64_encode(QrCode::format('png')->size(200)->generate(route('students.show', $student->id)));
        return view('students.show', compact('student','qrCode'));
    }

    public function edit(Student $student)
    {
        return view('students.edit', compact('student'));
    }

    public function update(Request $request, Student $student)
{
    $validated = $request->validate([
        'name' => 'required|string',
        'email' => 'required|email|unique:students,email,' . $student->id,
        'photo' => 'image|mimes:jpeg,png,jpg|max:2048',
        'passport_photo' => 'mimes:jpeg,png,jpg,pdf|max:2048',
    ]);

    // Initialize the data array with the fields that will always be updated
    $data = [
        'name' => $validated['name'],
        'email' => $validated['email'],
        'hotel_name' => $request->hotel_name,
        'university_name' => $request->university_name,
        'conference_date' => $request->conference_date,
    ];

    // Add 'photo' to the data array if a new photo was uploaded
    if ($request->hasFile('photo')) {
        $photoPath = $request->file('photo')->store('photos', 'public');
        $data['photo'] = $photoPath;
    }

    // Add 'passport_photo' to the data array if a new passport photo was uploaded
    if ($request->hasFile('passport_photo')) {
        $passportPhotoPath = $request->file('passport_photo')->store('passport_photos', 'public');
        $data['passport_photo'] = $passportPhotoPath;
    }

    // Update the student record
    $student->update($data);

    return redirect()->route('admin.students.index')->with('success', 'Student updated successfully.');
}


    public function destroy(Student $student)
    {
        if ($student->photo) {
            Storage::delete('public/' . $student->photo);
        }

        if ($student->passport_photo) {
            Storage::delete('public/' . $student->passport_photo);
        }

        $student->delete();
        return redirect()->route('admin.students.index')->with('success', 'Student deleted successfully.');
    }

//     public function sendEmailToStudent($student)
//     {
//         $link = route('students.show', $student->id);  // Replace this with your link
//         // Define the path to save the QR code
// $directory = public_path('qr-codes'); // Uses Laravel's public_path helper
// $filePath = $directory . '/'.$student->id ;

// // Create the directory if it does not exist
// if (!file_exists($directory)) {
//     mkdir($directory, 0755, true); // 0755 is the permission set for the directory
// }
//         $qrCode = new QrCode($link);
//         $writer = new PngWriter();
//         $result = $writer->write($qrCode);
        
//         // Save the QR code to a file
//         $filePath = public_path('qr-codes/student-' . $student->id . '.png');
//     file_put_contents($filePath, $result->getString());

//         Mail::send('emails.student_registration', compact('student', 'qrCode'), function($message) use ($student) {
//             $message->to($student->email);
//             $message->subject('Registration Confirmation');
//         });
//     }

public function sendEmailToStudent($student)
{
    // Step 1: Generate QR Code
    $qrCodePath = $this->generateQRCode($student);
    
    // Step 2: Generate .pkpass File
    $pkpassPath = $this->generatePassFile($student, $qrCodePath);
    $googlePath=$this->generateGooglePassFile($student);
    //dd($googlePath);
    // Step 3: Send Email with .pkpass Attachment
    $this->sendEmail($student, $qrCodePath, $pkpassPath,$googlePath);
}

    public function generateQRCode($student)
{
    $link = route('students.show', $student->id);  // Replace this with your link
    //         // Define the path to save the QR code
     $directory = public_path('qr-codes'); // Uses Laravel's public_path helper
    $filePath = $directory . '/'.$student->id ;
    
    // // Create the directory if it does not exist
     if (!file_exists($directory)) {
        mkdir($directory, 0755, true); // 0755 is the permission set for the directory
    }
            $qrCode = new QrCode($link);
            $writer = new PngWriter();
            $result = $writer->write($qrCode);
            
            // Save the QR code to a file
            $filePath = public_path('qr-codes/student-' . $student->id . '.png');
        file_put_contents($filePath, $result->getString());

    return $filePath;
}
public function generatePassFile($student, $qrCodePath)
{
    $pass = new PKPass(public_path('certificates/Certificates.p12'), 'Kanna5846iOSDev@');

// Set the pass data
$pass->setData([
    'description' => 'Event Ticket',
    'formatVersion' => 1,
    'organizationName' => 'Your Company',
    'passTypeIdentifier' => 'pass.loyaltycoupon.com',
    'serialNumber' => '123456',
    'teamIdentifier' => 'J69AR6Z6D9',
    'barcode' => [
        'message' => 'QR Code Data',
        'format' => 'PKBarcodeFormatQR',
        'messageEncoding' => 'iso-8859-1'
    ],
    'eventTicket' => [
        'primaryFields' => [
            [
                'key' => 'event',
                'label' => 'Event Name',
                'value' => 'Concert 2024'
            ]
        ]
    ]
]);

$pass->addRemoteFile(url("/qr-codes/student-$student->id.png")); // Icon for the pass
$pass->addFile(public_path("qr-codes/icon.png"));

// Generate the pass
$pkpass = $pass->create();

$passDirectory = public_path('passes');
    
    // Create directory if not exists
    if (!file_exists($passDirectory)) {
        mkdir($passDirectory, 0755, true);
    }

    $pkpassPath = $passDirectory . '/student-' . $student->id . '.pkpass';
    $pkpassData = $pass->create();

    if ($pkpassData === false) {
        // Handle the error appropriately
        throw new \Exception('Failed to generate the pass.');
    }
    // Save the .pkpass file
    file_put_contents($pkpassPath, $pkpass);

    return $pkpassPath;
}
public function sendEmail($student, $qrCodePath, $pkpassPath,$googlePath)
{
    Mail::send('emails.student_registration', compact('student','googlePath'), function($message) use ($student, $pkpassPath) {
        $message->to($student->email);
        $message->subject('Registration Confirmation');
        
        // Attach the .pkpass file
        $message->attach($pkpassPath, [
            'as' => 'event-ticket.pkpass',
            'mime' => 'application/vnd.apple.pkpass'
        ]);
    });
}


    public function generateWallet(Student $student)
    {
        $this->getAccessToken($student);
        // Logic for generating a Pass file for Apple Wallet/Google Wallet can go here.
        return back()->with('info', 'Wallet functionality is under development.');
    }
    public function getApplePass($student){
       
        

        $pass = new PKPass(public_path('certificates/Certificates.p12'), 'Kanna5846iOSDev@');

// Set the pass data
$pass->setData([
    'description' => 'Event Ticket',
    'formatVersion' => 1,
    'organizationName' => 'Your Company',
    'passTypeIdentifier' => 'pass.loyaltycoupon.com',
    'serialNumber' => '123456',
    'teamIdentifier' => 'J69AR6Z6D9',
    'barcode' => [
        'message' => 'QR Code Data',
        'format' => 'PKBarcodeFormatQR',
        'messageEncoding' => 'iso-8859-1'
    ],
    'eventTicket' => [
        'primaryFields' => [
            [
                'key' => 'event',
                'label' => 'Event Name',
                'value' => 'Concert 2024'
            ]
        ]
    ]
]);

$pass->addRemoteFile(url("/qr-codes/student-$student->id.png")); // Icon for the pass
$pass->addFile(public_path("qr-codes/icon.png"));

// Generate the pass
$pkpass = $pass->create();

if ($pkpass === false) {
    echo "Error generating the pass.";
    exit;
}

// Set the headers for file download
header('Content-Type: application/vnd.apple.pkpass');
header('Content-Disposition: attachment; filename="pass.pkpass"');
header('Content-Transfer-Encoding: binary');
header('Content-Length: ' . strlen($pkpass));

// Clear output buffer
if (ob_get_length()) {
    ob_end_clean();
}

// Output the .pkpass file
echo $pkpass;

        
dd($pass);
        $pass = new PKPass();
        
        // Define the path to your certificates and key files
        $passCertificate = public_path('certificates/passCertificate.pem'); // Pass Type ID Certificate
        $privateKey = public_path('certificates/privateKey.pem'); // Private Key
        $wwdrCertificate = public_path('certificates/wwdr.pem'); // WWDR Intermediate Certificate
        $password = 'Kanna5846iOSDev@'; // Password for private key
        
        // Pass JSON data (example pass structure)
        $passData = [
            'description' => 'Example Pass',
            'formatVersion' => 1,
            'organizationName' => 'Your Organization',
            'passTypeIdentifier' => 'pass.com.yourorganization.pass',
            'serialNumber' => '123456',
            'teamIdentifier' => 'TEAMID',
            // Add other necessary fields here...
        ];
        
        // Add pass data to the PKPass instance
        $pass->setJSON(json_encode($passData));
        
        // Create the pass using the certificates
        $pkpass = $pass->create($passCertificate, $privateKey, $password, $wwdrCertificate);
        
        // Set headers for pass download
        header('Content-Type: application/vnd.apple.pkpass');
        header('Content-Disposition: attachment; filename="pass.pkpass"');
        
        // Output the pass package
        echo $pkpass;
dd('there');        
// Add pass data
$pass->setJSON(json_encode($passData));

// Create the pass
$pass->create();
dd($pass);
// Define the pass type (coupon, eventTicket, storeCard, etc.)
$pass->setType('eventTicket'); // Choose appropriate type

// Set JSON structure for the pass
$pass->setJson('{
    "description": "Event Ticket Pass",
    "formatVersion": 1,
    "organizationName": '.$student->university_name.',
    "passTypeIdentifier": "pass.loyaltycoupon.com",
    "serialNumber": '.$student->universiidy_name.',
    "teamIdentifier": "J69AR6Z6D9",
    "backgroundColor": "rgb(255,255,255)",
    "labelColor": "rgb(255,255,255)",
    "foregroundColor": "rgb(0,0,0)",
    "barcode": {
        "message": "https://example.com/qr-code",
        "format": "PKBarcodeFormatQR",
        "messageEncoding": "iso-8859-1"
    },
    "organizationName": "Your Organization"
}');

// Add images (logo, icon, etc.)
$pass->addRemoteFile(url("/qr-codes/student-$student->id")); // Icon for the pass
//$pass->addFile('logo.png'); // Logo for the pass

// Create and sign the pass
$pass->create($passCertificate, $privateKey, $wwdrCertificate, $password);

// Output the .pkpass file
header('Content-Type: application/vnd.apple.pkpass');
header('Content-Disposition: attachment; filename="pass.pkpass"');
echo $pass->output();

    }

   

public function generateGooglePassFile($student){

// Google Service Account JSON Key
$serviceAccountKey = json_decode(file_get_contents(public_path('certificates/even-crawler.json')), true);

// Wallet Class and Object Identifiers
$classId = '3388000000022748401';
$objectId = 'studentcardsapi';

// Create the JWT Payload
$jwtPayload = [
    'iss' => $serviceAccountKey['client_email'],
    'aud' => 'google',
    'typ' => 'savetowallet',
    'payload' => [
        'eventTicketObjects' => [
            [
                'id' => $objectId,
                'classId' => $classId,
                'heroImage' => [
                    'sourceUri' => [
                        'uri' => 'https://example.com/your-image.png',
                    ],
                    'contentDescription' => [
                        'defaultValue' => [
                            'language' => 'en',
                            'value' => 'Hero image description',
                        ],
                    ],
                ],
                'barcode' => [
                    'type' => 'qrCode',
                    'value' => url("/qr-codes/student-$student->id.png"), // Your QR Code link or text
                    'alternateText' => 'Show this QR code at the event',
                ],
                'state' => 'active',
                'eventName' => [
                    'defaultValue' => [
                        'language' => 'en',
                        'value' => 'Your Event Name',
                    ],
                ],
                'seatInfo' => [
                    'seat' => [
                        'defaultValue' => [
                            'language' => 'en',
                            'value' => 'Row 10, Seat 5',
                        ],
                    ],
                    'section' => [
                        'defaultValue' => [
                            'language' => 'en',
                            'value' => 'Section A',
                        ],
                    ],
                    'gate' => [
                        'defaultValue' => [
                            'language' => 'en',
                            'value' => 'Gate 1',
                        ],
                    ],
                    'venue' => [
                        'name' => [
                            'defaultValue' => [
                                'language' => 'en',
                                'value' => 'Event Venue Name',
                            ],
                        ],
                        'address' => [
                            'defaultValue' => [
                                'language' => 'en',
                                'value' => '123 Event Street, City, Country',
                            ],
                        ],
                    ],
                ],
                'validTimeInterval' => [
                    'start' => [
                        'date' => '2024-09-30T10:00:00Z',
                    ],
                    'end' => [
                        'date' => '2024-09-30T23:00:00Z',
                    ],
                ],
            ],
        ],
    ],
];

// Sign the JWT using the service account private key
$privateKey = $serviceAccountKey['private_key'];
$jwtToken = JWT::encode($jwtPayload, $privateKey, 'RS256');

// Create the "Save to Google Wallet" URL
$saveToWalletUrl = 'https://pay.google.com/gp/v/save/' . $jwtToken;

return $saveToWalletUrl;
}

private function getAccessToken()
    {


putenv("GRPC_SSL_CIPHER_SUITES=HIGH+ECDSA");
// create-event takes the venueId and creates an event for an event ticket.
try {
    
  
    $cert_filename  = public_path('certificates/passCertificate.pem'); // Pass Type ID Certificate
    $key_filename= public_path('certificates/privateKey.pem'); // Private Key
    $ca_filename = public_path('certificates/ca-chain.pem');

    $credentials = \Grpc\ChannelCredentials::createSsl(
        file_get_contents($ca_filename),
        file_get_contents($key_filename),
        file_get_contents($cert_filename)
    );

    //Create events client
    $eventsclient = new Event_tickets\EventTicketsClient('grpc.pub1.passkit.io:443', [
        'credentials' => $credentials
    ]);
    // Create a new production
try {
    $production = new Production();
    $production->setName("My Production");
    $production->setDescription("Description of the event series or production");

    // Create the production using the client
    list($newProduction, $status) = $eventsclient->createProduction($production)->wait();

    if ($status->code !== 0) {
        throw new Exception(sprintf('Error creating production: %s', $status->details));
    }

    // Get the Production Id
    $productionId = $newProduction->getId();
    echo "Production Id: " . $productionId . "\n";

} catch (Exception $e) {
    echo "Error: " . $e->getMessage();
}

// Create a new venue
try {
    $venue = new Venue();
    $venue->setName("Venue Name");
    $venue->setAddressLine1("123 Main St.");
    $venue->setCity("City Name");
    $venue->setState("State Name");
    $venue->setCountry("Country Name");

    // Create the venue using the client
    list($newVenue, $status) = $eventsclient->createVenue($venue)->wait();

    if ($status->code !== 0) {
        throw new Exception(sprintf('Error creating venue: %s', $status->details));
    }

    // Get the Venue Id
    $venueId = $newVenue->getId();
    echo "Venue Id: " . $venueId . "\n";

} catch (Exception $e) {
    echo "Error: " . $e->getMessage();
}


    $production = new Production();
    $production->setId($productionId );
    $venue = new Venue();
    $venue->setId($venueId );

    $startDate = new DateTime();
    $startDate->setDate(2023, 12, 12);
    $startDate->setTime(13, 0, 0);
    $startDate = $startDate->getTimestamp();
    $endDate = new DateTime();
    $endDate->setDate(2023, 12, 13);
    $endDate->setTime(13, 0, 0);
    $endDate = $endDate->getTimestamp();
    $doorsOpen = new DateTime();
    $doorsOpen->setDate(2023, 12, 13);
    $doorsOpen->setTime(14, 0, 0);
    $doorsOpen = $doorsOpen->getTimestamp();

    // Create the event for the event ticket
    $event = new Event();
    $event->setProduction($production);
    $event->setVenue($venue);
    $event->setScheduledStartDate($startDate);
    $event->setDoorsOpen($doorsOpen);
    $event->setEndDate($endDate);
    $event->setRelevantDate($startDate);

    list($id, $status) = $eventsclient->createEvent($event)->wait();
    if ($status->code !== 0) {
        throw new Exception(sprintf('Status Code: %s, Details: %s, Meta: %s', $status->code, $status->details, var_dump($status->metadata)));
    }

    //You can use the eventId displayed below for other event ticket methods
    echo "EventId: " . $id->getId() . "\n";
} catch (Exception $e) {
    echo $e;
}

    }

}
